@@ -35,6 +35,7 @@ root of all actors, all modules to be used within this bot have to inherit from
 ##############################################################################
 package zebot::baseactor;
 use strict;
+use Encode 'from_to';
 
 sub BEGIN
 {
@@ -86,7 +87,28 @@ sub init
   my @classSplit = split(/::/,ref($this));
   #print("plit is ".Dumper(\@classSplit)."\n");
   my $rawname = pop(@classSplit);
-  $this->{"subsettings"} = $botref->{"settings"}->{"subsettings"}->{$rawname};
+  #lower classes may override setting to make special stuff, let them parse the
+  #config files altogether
+  #$this->{"subsettings"} = $botref->{"settings"}->{"subsettings"}->{$rawname};
+
+  my $subset = $botref->{"settings"}->{"subsettings"}->{$rawname};
+  foreach my $key (keys(%{$subset}))
+  {
+    $this->setting($key,$subset->{$key});
+  }# foreach my $key (keys(%{$subset}))
+
+  #load eventual text ressources needed by this module
+  my $path = $this->setting("configpath")."ressources/";
+  if($this->setting("messages"))
+  {
+    $path .= $this->setting("messages") ;
+  }
+  else
+  {
+    $path .= "res_".$this->isa().".xml" ;
+  }# else
+  my $msgHand = $botref->module("messageHandler");
+  $msgHand->load($path) if(-e $path);
   #print($this->isa()." init with settings:".Dumper($this->{"subsettings"})."\n");
   # $this->print("baseactor::init bot? ".Dumper($botref));
   #POE::Session->new( $this->isa() =>
@@ -131,6 +153,13 @@ sub activeForThisChan
   my $channel = $args->{"channel"};
   my $heap = $args->{"heap"};
   # $this->print("baseactor::active bot? ".Dumper($this->{"zebot"}));
+  #print("START of heap dump\n");
+  #foreach my $key (%{$heap})
+  #{
+  #  print("heapvalue $key = ". $heap->{$key}."\n");
+  #}# foreach my $key (%{$heap})
+  #print("end of heap dump\n");
+
   my $botname = $heap->{"nick"};
   if(!$botname)
   {
@@ -149,11 +178,51 @@ sub activeForThisChan
   #$this->print("checking for botname?\n");
   return 0 if($botname eq $channel);
   #$this->print("checking for presence in ".Dumper($heap)."?\n");
-  return 0 if($heap->{"channels"}->{lc($channel)});
+  #return 0 if($heap->{"channels"}->{lc($channel)});
+  return 0 if($heap->{"actives"}->{lc($channel)}->{"all"});
+  return 0 if($heap->{"actives"}->{lc($channel)}->{$this->isa()});
   #return 0 if($activeChans =~ /\Q$channel\E/i);
   #$this->print("seems this channel is unknwon...?\n");
   return 1;
 }# sub activeForThisChan
+#########################################################
+=pod
+
+=item activate
+
+make this module active for a given channel
+
+=cut
+
+#########################################################
+sub activate
+{
+  my ($this,$args) = @_;
+  my $channel = $args->{"channel"};
+  my $lang = $args->{"lang"};
+  my $heap = $args->{"heap"};
+
+  $heap->{"actives"}->{lc($channel)}->{$this->isa()} = 1;
+}# sub activate
+#########################################################
+=pod
+
+=item deactivate
+
+stop this module for a given channel
+
+=cut
+
+#########################################################
+sub deactivate
+{
+  my ($this,$args) = @_;
+  my $channel = $args->{"channel"};
+  my $lang = $args->{"lang"};
+  my $heap = $args->{"heap"};
+
+  delete($heap->{"actives"}->{lc($channel)}->{$this->isa()});
+}# sub deactivate
 ##############################################################################
 =pod
 
@@ -372,7 +441,7 @@ issue the version of this module
 ##############################################################################
 sub version
 {
-  return '$Revision: 1.29 $';
+  return '$Revision: 1.34 $';
 }#sub isa
 ##############################################################################
 =pod
@@ -390,26 +459,49 @@ sub post
   my ($this,$nick, $msg) = @_;
   my $kernel = $this->{"kernel"};
   my $context = $this->{"context"};
-  #$this->print( "pam::post kernel=". $this->{"kernel"}."\n" );
+  #$this->print( "basactor::post kernel=". $this->{"kernel"}."\n" );
+  #$this->print( "basactor::post context='$context'" );
+  my $mode = 'privmsg';
 
-  my @text = split(/\n/,$msg);
-  if($kernel)
+  if($context)
   {
-    foreach my $line (@text)
+    #backpropagation!! we asked inside the program to send us back something..
+    if($context =~ /\@/)
     {
-      if($nick && $line)
+      ($mode, $context) = split(/\@/,$context);
+      #print("posting with remaining args:".Dumper(\@_)."\n");
+      if(ref($msg) ne "HASH")
       {
-	$kernel->post( $context, 'privmsg', $nick,$line);
-      }# if($nick && $line)
-      else
+	print("cretin oublie de changer l'envoi de msg de $msg!!\n");
+      }# if(ref($msg) ne "HASH")
+      $kernel->post( $context, $mode, $msg);
+      return;
+    }
+  }# if($context =~ /@/)
+
+  #ircmode, we don't need the rest of the info
+      $msg = $msg->{"line"} if(ref($msg) eq "HASH");
+
+    my @text = split(/\n/,$msg);
+    if($kernel)
+    {
+      foreach my $line (@text)
       {
-	 $this->print("oy post needs 2 args, org msg was: '$msg' act line was u:l->'$nick':'$line'\n") if(!$msg);
-      }
-    }# foreach my $line (@text)
-    #$this->print("issuing post to $context with 'privmsg' and $nick soste $msg\n");
-  }#if ($kernel)
-  else
-  {$this->print("$nick $msg\n");}
+
+	#from_to($line, "utf-8","iso-8859-1");
+	if($nick && $line)
+	{
+	  $kernel->post( $context, $mode, $nick,$line);
+	}# if($nick && $line)
+	else
+	{
+	  $this->print("oy post needs 2 args, org msg was: '$msg' act line was u:l->'$nick':'$line'\n") if(!$msg);
+	}
+      }# foreach my $line (@text)
+      #$this->print("issuing post to $context with 'privmsg' and $nick soste $msg\n");
+    }#if ($kernel)
+    else
+    {$this->print("$nick $msg\n");}
   return 0;
 }
 ##############################################################################
@@ -552,7 +644,12 @@ sub action
 
   if ($kernel)
   {
-    $kernel->post( $context, 'privmsg', $nick,'ACTION '.$msg.'');
+    my @lines = split(/\n/,$msg);
+    foreach my $line (@lines)
+    {
+      #from_to($line, "utf-8","iso-8859-1");
+      $kernel->post( $context, 'privmsg', $nick,'ACTION '.$line.'');
+    }
   }#if ($kernel)
   else
   {$this->print( "ACTION $nick $msg\n");}
@@ -743,8 +840,14 @@ help accessor
 ##############################################################################
 sub help
 {
-  my $this = shift;
-  return $this->isa().":no help available\n";
+  my ($this,$line) = @_;
+  my $msgHand = ($this->sysdata())->module("messageHandler");
+  my $token = uc($this->isa());
+  $token .= "HELP";
+
+  my $msg = $msgHand->getMesg($token,$line);
+  return $msg if($msg);
+  return $this->isa().$msgHand->getMesg(":no help available",$line)."\n";
 }#sub help
 #########################################################
 =pod
@@ -984,8 +1087,8 @@ its tested if there are some access rights fr that user, and if they correspond
 sub isOwnerOrOper 
 {
   my ($this, $subargs) = @_;
-    my $pam = ($this->sysdata())->{"actors"}->{"pam"};
-    my $accessRights = $pam->findOperator($subargs);
+  my $pam = ($this->sysdata())->module("pam");
+  my $accessRights = $pam->findOperator($subargs);
 
   return 1    if($accessRights && (index($accessRights,"owner") != -1 || index($accessRights,"oper") != -1));
   return 0;
@@ -1030,6 +1133,18 @@ sub setting
 
   my $botref =  $this->{"zebot"};
 
+  if(!$key)
+  {
+    #make a dump of the settings
+    my $msg = "Settings of Module ".$this->isa()."\n";
+    foreach $key (sort(keys(%{$this->{"subsettings"}})))
+    {
+     $msg .= "$key: ".$this->{"subsettings"}->{$key}."\n"
+         if(ref($this->{"subsettings"}->{$key}) eq ""); 
+    }# foreach $key (keys(%$this))
+    return($msg);
+  }# if(!$key)
+
   if($value)
   {
     if($botref->setting($key))
