@@ -114,11 +114,28 @@ sub init
 
   foreach my $canal (@Chans)
   {
-    $canal =~ /(.*?)%(.*?)%/;
-    $canal = $1;
-    my $lang = $2;
-    $botref->{"channelLangs"}->{"$canal"} = $lang;
+    my @fields = split(/\%/,$canal);
+    #$canal =~ /(.*?)%(.*?)%(.*)/;
+    #$canal = $1;
+    #my $lang = $2;
+    #my $modules = $3;
+    $canal = shift(@fields);
+    my $lang = shift(@fields);
+    $botref->language($canal,$lang);
     $this->{"channelLangs"}->{lc($canal)} = $lang;
+    my $modules = shift(@fields);
+    if($modules)
+    {
+      my @actives = split(/\+/,$modules);
+      foreach my $mod (@actives)
+      {
+	$this->{"activeMod"}->{lc($canal)}->{$mod} = 1;
+      }# foreach my $mod (@actives)
+    }# if($modules)
+    else
+    {
+	$this->{"activeMod"}->{lc($canal)}->{"all"} = 1;
+    }# else
     #$heap->{"channels"}->{"$canal"} = $lang;
     #print("SET lang $lang as default lang of $canal: dumping channel languages: ".Dumper($botref->{"channelLangs"}));
 
@@ -213,6 +230,7 @@ sub _start
   {
     $heap->{"channels"}->{"$canal"} = $this->{"channelLangs"}->{"$canal"};
     #print("SET lang $lang as default lang of $canal: dumping channel languages: ".Dumper($botref->{"channelLangs"}));
+	$heap->{"actives"} = $this->{"activeMod"};
 
   }# foreach my $canal (@Chans)
   # Uncomment this to turn on more verbose POE debugging information.
@@ -396,7 +414,7 @@ return the version of this module
 #########################################################
 sub version
 {
-    return '$Revision: 1.17 $';
+    return '$Revision: 1.20 $';
 }#sub isa
 #########################################################
 =pod
@@ -650,7 +668,7 @@ sub rejoin
     if($nick)
     {
     #my $actLang = $args->{"lang"};
-      my $msgHand = $this->{"zebot"}->{"actors"}->{"messageHandler"};
+    my $msgHand = ($this->{"zebot"})->module("messageHandler");
       my $message = $msgHand->getMesg("MORDS",$args);
       $message =~ s/%target/$nick/g;
       $kernel->post( $context, 'privmsg', $addressed, $message);
@@ -805,9 +823,40 @@ settings without disturbing the rest of the program...
 #########################################################
 sub setting 
 {
-  my $this = shift(@_);
+  my ($this,$key,$value) = @_;
+
   my $botref =  $this->{"zebot"};
-  return $botref->setting(@_);
+
+  if(!$key)
+  {
+    #make a dump of the settings
+    my $msg = "Settings of ircServer ".$this->{"name"}."\n";
+    foreach $key (sort(keys(%$this)))
+    {
+     $msg .= $key.": ".$this->{$key}."\n" if($this->{$key} && ref($this->{$key}) eq ""); 
+    }# foreach $key (keys(%$this))
+    $msg .= "End of listing\n";
+    return($msg);
+  }# if(!$key)
+
+  if($value)
+  {
+    if($botref->setting($key))
+    {
+      #its a global setting 
+      return $botref->setting($key,$value);
+    }# if($botref->setting($key))
+    #local setting
+    $this->{$key} = $value;
+    return;
+  }# if ($value)
+
+  if($this->{$key})
+  {
+    return $this->{$key};
+  }# if ($key)
+
+  return $botref->setting($key,$value);
 }#setting
 #########################################################
 =pod
@@ -821,23 +870,36 @@ someone invited us, be so nice as to join their channel
 #########################################################
 sub irc_invite
 {
-  my ($this,$usernick,$channel, $kernel,$heap,$context ) 
-    = @_[OBJECT, ARG0 .. ARG1,KERNEL,HEAP,SENDER];
-    $kernel->post($context, 'join', $channel );
-    $kernel->post($context,'privmsg', $channel,$this->{"zebot"}->getMesg("CHANJOIN"));
+  my ($this,$usernick,$channel, $kernel,$heap,$context ) = @_[OBJECT, ARG0 .. ARG1,KERNEL,HEAP,SENDER];
+  $kernel->post($context, 'join', $channel );
+  $kernel->post($context,'privmsg', $channel,$this->{"zebot"}->getMesg("CHANJOIN"));
   my $actLang = $heap->{"channels"}->{"$channel"};
   $actLang = ($this->{"zebot"})->setting("language") if(!$actLang);
+  my ($nick,$name,$host) = ( $usernick =~ /(\S+)!(\S+)@(\S+)/);
+  my $botref = $this->{"zebot"};
+  my $pam = $botref->module("pam");
+  my $count = $pam->countRegisteredUsers($heap->{"network"},$channel);
 
-    my $subargs = {
-      "heap" => $heap,
-      "usernick" =>$usernick,
-      "channel" =>$channel, 
-      "lang" => $actLang,
-      "network" => $heap->{"network"},
-      "context" =>$this->{"name"}, 
-    };
+  my $subargs = {
+    "heap" => $heap,
+    "usernick" =>$nick,
+    "channel" =>$channel, 
+    "lang" => $actLang,
+    "network" => $heap->{"network"},
+    "context" =>$this->{"name"}, 
+    "kernel" => $kernel,
+  };
 
   $kernel->post('mainframe', "irc_invite",$subargs);
+
+  if($count == 0)
+  {
+    #try to give this lone user some op rights...
+    $heap->{"actives"}->{$channel}->{"pam"} = 1;
+    $heap->{"actives"}->{$channel}->{"basereactions"} = 1;
+    $pam->join_success($subargs);
+    $pam->addToGroup($subargs,"oper",$nick);
+  }# if($count == 0)
 }#sub irc_invite
 #########################################################
 =pod
@@ -1337,7 +1399,9 @@ sub channel_join_topic
   #my ($nick,$name,$host) = ( $usernick =~ /(\S+)!(\S+)@(\S+)/);
   my @arguments = @_[ARG0..ARG9];
 
-  print("channel_join_topic unnkown args:".Dumper(\@arguments)."\n");
+  #print("channel_join_topic unnkown args:".Dumper(\@arguments)."\n");
+  my $servername = shift(@arguments);
+  my $channelTopic = shift(@arguments);
 
   #my $actLang = $heap->{"channels"}->{$addressed->[0]};
   my $actLang ;
@@ -1348,6 +1412,8 @@ sub channel_join_topic
     "lang" => $actLang,
       "network" => $heap->{"network"},
       "context" =>$this->{"name"}, #this one holds the link ot the actual server, can be used as post target
+	"servername"	=> $servername,
+	"topic"	=> $channelTopic,
   };
 
   #trying to fetch the list of people
