@@ -43,20 +43,15 @@ ressource name doesn't begin with a /
 package zebot::messageHandler;
 use strict;
 use zebot::baseactor;
+use zebot::emoteparser;
 #use ObjectTemplate;
 use POSIX qw(strftime);
 
 use Data::Dumper;
 
-# sub POE::Kernel::ASSERT_DEFAULT () { 1 }
-# sub POE::Kernel::ASSERT_PONY () { 1 }
-# sub POE::Kernel::ASSERT_EVENTS () { 1 }
-# sub POE::Kernel::ASSERT_STATES () { 1 }
-
 use POE::Session;
 
 our @ISA = ("zebot::baseactor");
-#attributes("users","deb","active","errmsg","dbh");
 
 zebot::baseactor::_define_constructor("zebot::messageHandler");
 
@@ -79,8 +74,6 @@ sub init
   $this->setting("xmlreactionsfile","res_emotes.xml") if(!($this->setting("xmlreactionsfile"))); 
   $this->setting("xmlmessagesfile","res_messages.xml") if(!($this->setting("xmlmessagesfile"))); 
 
-  $this->{"tags"} = {};
-  $this->{"langs"} = {};
 
   #$this->print "load reactions: ".$this->setting("xmlreactionsfile")."\n";
 
@@ -125,7 +118,6 @@ sub help
   #my $userhost = $splittedline->{"userhost"};
   #my $kernel = $splittedline->{"kernel"};
 
-  #$actLang = ($this->sysdata())->{"channelLangs"}->{"$channel"};
   #$this->print("trying to fetch lang $actLang from chan $channel\n");
   #$actLang = "en" if(!$actLang);
   #my $actLang = $this->fetchChannelLang($channel);
@@ -176,7 +168,7 @@ return the version of this module
 ######################################################################
 sub version
 {
-  return '$Revision: 1.14 $';
+  return '$Revision: 1.20 $';
 }#sub isa
 #########################################################
 =pod
@@ -191,14 +183,16 @@ load the file with the basic reactions of the bot
 sub load 
 {
   my ($this,$line) = @_;
+  $this->{"tags"} = {} if(!$this->{"tags"});
+  $this->{"langs"} = {} if(!$this->{"langs"});
   my $TokTree = $this->{"tags"};
   my $LanTree = $this->{"langs"};
 
-  use zebot::emoteparser;
   my $emoteparser = new zebot::emoteparser();
 
   $emoteparser->init();
 
+  $this->print("Official reactions parsing $line!! \n");
   $emoteparser->parse($line,$TokTree,$LanTree);
   #$this->print "Official reactions!! \n";
   #$this->print dumpLangs();
@@ -271,32 +265,55 @@ sub dumpXML
 {
   my ($this) = @_;
   my $i;
-  my $msg = {"emotes" => "<emotes>\n", "mesg" => "<emotes>\n"};
+  my $msg = {"emotes" => "", "mesg" => ""};
 
-  my $actRef = "";
+  my $visited = {};
 
-    my $ressource = $this->{"tags"};
-   foreach $i (sort(keys(%$ressource)))
-   {
-     my $line = $ressource->{$i};
-     my $entry  = "<emote tag=\"$i\"";
-     $entry .= " active=\"".$line->{"active"}."\"" if($line->{"active"});
-     $entry .= ">\n";
-     $entry .= $this->dumpEmote($line);
-     $entry .= "</emote>\n";
-     if($line->{"active"})
-     {
-       $msg->{"emotes"} .= $entry;
-     }# if($line->{"active"})
-     else
-     {
-       $msg->{"mesg"} .= $entry;
-     }# else
-   }# foreach $i (keys(%theActions))
-   $msg->{"emotes"} .= "</emotes>";
-   $msg->{"mesg"} .= "</emotes>";
-  return $msg->{"emotes"}."\nSTART OF MESSAGES\n".$msg->{"mesg"};
-}# sub dump 
+  my $ressource = $this->{"tags"};
+  
+  my @allkeys = sort(keys(%$ressource));
+  my $allactors = ($this->sysdata())->module();
+  foreach my $actor (sort(keys(%$allactors)))
+  {
+    my $key = uc($actor);
+    my @actorKeys = grep { /^$key/ }@allkeys;
+    my $listing;
+    foreach my $actorkey (sort(@actorKeys))
+    {
+      $visited->{$actorkey}++;
+      my $line = $ressource->{$actorkey};
+      $listing .= $this->emoteToXml($actorkey,$line)."\n";
+    }# foreach my $actorkey (sort(@actorKeys))
+    $msg->{$actor} = $listing;
+  }# foreach my $actor (sort(keys(%$allactors)))
+
+  foreach $i (sort(keys(%$ressource)))
+  {
+    next if($visited->{$i});
+    my $line = $ressource->{$i};
+    my $entry = $this->emoteToXml($i,$line);
+    if($line->{"active"})
+    {
+      $msg->{"emotes"} .= $entry;
+    }# if($line->{"active"})
+    else
+    {
+      $msg->{"mesg"} .= $entry;
+    }# else
+  }# foreach $i (keys(%theActions))
+
+  foreach my $name (keys(%$msg))
+  {
+    if($msg->{$name})
+    {
+      my $finalencode = '<?xml version="1.0" encoding="ISO-8859-1" ?>';
+      $finalencode .=  "\n<emotes>\n".$msg->{$name}."</emotes>";
+      $msg->{$name} = $finalencode;
+    }
+  }# foreach my $module (keys(%$msg))
+
+  return $msg;
+}# sub dumpXML 
 #########################################################
 =pod
 
@@ -405,16 +422,39 @@ sub save
 {
   my ($this) = @_;
   my $datapack = $this->dumpXML();
-  my($emotes,$messages) = split(/START OF MESSAGES/,$datapack);
+  my %nameToFile = ();
+  my $root = $this->setting("configpath")."ressources/"; 
+
+  foreach my $name (keys(%$datapack))
+  {
+    if($name eq "emotes")
+    {
+      $nameToFile{$name} = $root.$this->setting("xmlreactionsfile");
+    }# if($name eq "emotes")
+    elsif($name eq "mesg")
+    {
+      $nameToFile{$name} = $root.$this->setting("xmlmessagesfile");
+    }# elsif($name eq "mesg")
+    else
+    {
+      my $filename = $this->sysdata()->module($name)->setting("messages");
+      $filename = "res_$name.xml" if(!$filename);
+      $nameToFile{$name} = $root.$filename;
+    }# else
+  }# foreach my $name (keys(%$datapack))
+  
   #$this->print("messageHandler::shutdown saving the stuff: $datapack\n");
   #$this->post( $usernick,"messageHandler::shutdown saving the stuff: $datapack\n");
   #$this->print("messageHandler::shutdown saving emotes to : ".$this->setting("xmlreactionsfile")."\n");
-  #$this->post( $usernick,"messageHandler::shutdown saving emotes to : ".$this->setting("xmlreactionsfile"));
-  my $root = $this->setting("configpath")."ressources/"; 
-  $this->saveToFile($root.$this->setting("xmlreactionsfile"),$emotes);
-  #$this->print("messageHandler::shutdown saving messages to : ".$this->setting("xmlmessagesfile")."\n");
-  #$this->post( $usernick,"messageHandler::shutdown saving messages to : ".$this->setting("xmlmessagesfile"));
-  $this->saveToFile($root.$this->setting("xmlmessagesfile"),$messages);
+
+  foreach my $name (keys(%nameToFile))
+  {
+    #$this->post( $usernick,"messageHandler::shutdown saving text ressources for $name  to : ".$nameToFile->{$name});
+    if($datapack->{$name})
+    {
+      $this->saveToFile($nameToFile{$name},$datapack->{$name});
+    }
+  }
   return 0;
 }# sub save
 #########################################################
@@ -460,8 +500,8 @@ sub getMesg
   my $actToken = $tokList->{$token};
   if(!$actToken)
   {
-   $this->print("messageHandler::getMsg nothing known about $token\n");
-   $this->print("messageHandler::getMsg known tokens: ".keys(%{$tokList})."\n");
+   #$this->print("messageHandler::getMsg nothing known about $token\n");
+   #$this->print("messageHandler::getMsg known tokens: ".keys(%{$tokList})."\n");
     #add an empty token to remind that it needs to be filled...
     my %subargs = (
     "lang"	=>	$lang,
@@ -551,15 +591,15 @@ sub getEmote
   my $actTriggers = $langList->{$lang};
   if(!$actTriggers)
   {
-   $this->print("messageHandler::getMsg nothing known about $lang\n");
+    $this->print("messageHandler::getMsg nothing known about $lang\n");
     return "";
   }# if(!($langList->{uc($trigger)}))
 
   my $actTrigger = $actTriggers->{$trigger};
   if(!$actTrigger)
   {
-   $this->print("messageHandler::getMsg nothing known about $trigger\n");
-   #$this->print("messageHandler::getMsg dumping available ones: ".Dumper($actTriggers)."\n");
+    $this->print("messageHandler::getMsg nothing known about $trigger\n");
+    #$this->print("messageHandler::getMsg dumping available ones: ".Dumper($actTriggers)."\n");
     return "";
   }# if(!($langList->{uc($trigger)}))
   #$this->print("dumping act trigger: ".Dumper($actTrigger)."\n");
@@ -571,12 +611,34 @@ sub getEmote
     #$this->print("dumping act language ".Dumper($actEntry)."\n");
 
     #last check if the given mode points to something, bail out if even the default one doesn't match
+    #$this->print("checking mode $mode... in entry: ".Dumper($actEntry->{"modes"})."\n");
     if(!($actEntry->{"modes"}->{$mode}))
     {
-      $mode = "default";
+      if($mode =~ /arg$/)
+      {
+	if($mode =~ /^user/i)
+	{
+	  $mode =~ s/user/default/;
+	  if(!($actEntry->{"modes"}->{$mode}))
+	  {
+	      $mode = "user" if($actEntry->{"modes"}->{"user"});
+	      $mode = "default" if(!($actEntry->{"modes"}->{"user"}));
+	  }#if(!($actEntry->{"modes"}->{$mode}))
+	}# if($mode ~= /^user/i)
+	else
+	{
+	  $mode =~ s/arg$//;
+	  $mode = "default" if(!($actEntry->{"modes"}->{$mode}));
+	}
+      }# if($mode =~ /arg$/)
+      else
+      {
+	$mode = "default";
+      }
       return "" if(!($actEntry->{"modes"}->{$mode}));
     }# if(!($actEntry->{"modes"}->{$mode}))
 
+
     my $actEmote = $actEntry->{"modes"}->{$mode};
     my $actReply = $actEmote->{"reply"};
     $actReply .= $actEmote->{"body"} if($actEmote->{"body"}); 
@@ -584,9 +646,9 @@ sub getEmote
     #$this->print("messageHandler::getEmote is satisfying: ".$actEmote->{"reply"}."\n");
     my $toReturn = $actEmote->{"reply"};
     $toReturn .= $actEmote->{"body"} if($actEmote->{"body"}); 
-  
-  $toReturn = $this->completeMsg($toReturn,$subargs);
-  $toReturn = "" if($toReturn eq 'NONE');
+
+    $toReturn = $this->completeMsg($toReturn,$subargs);
+    $toReturn = "" if(!$toReturn || $toReturn eq 'NONE');
     return $toReturn if($toReturn);
     #return $actEmote->{"reply"} if($actEmote->{"reply"});
     #hmmm lets check if this wasn't some action...
@@ -605,7 +667,7 @@ sub getEmote
   #$this->print("MESSAGEhANDLER::GETmSG dumping record ".Dumper($actEntry)."\n");
   #$this->print("MESSAGEhANDLER::GETmSG returning ".$actEntry->{$mode}->{"reply"}."\n");
   #return $actEntry->{$mode}->{"reply"} if($actEntry->{$mode}->{"reply"});
-  
+
   #return $default;
   return "";
 }# sub getEmote
@@ -654,7 +716,7 @@ sub PRIVMSGaction
     if($rest) {$line .= " ".$rest;}
   }# else
   $command = lc($command);
-  my $pam = ($this->sysdata())->{"actors"}->{"pam"};
+    my $pam = ($this->sysdata())->module("pam");
   my $accessRights = $pam->findOperator($splittedline);
   if($command =~ /addmsg/)
   {
@@ -745,8 +807,7 @@ small function that retrieves the language of a channel
 sub fetchChannelLang
 {
   my($this,$channel) = @_;
-  #$this->print("searching lang for chan $channel, in :".Dumper(($this->sysdata())->{"channelLangs"})."\n");
-  my $actLang = ($this->sysdata())->{"channelLangs"}->{"$channel"};
+  my $actLang = ($this->sysdata())->language($channel);
   $actLang = $this->setting("language") if(!$actLang);
   #$this->print("returning $actLang\n");
   return $actLang;
@@ -797,7 +858,7 @@ sub addEmote
   $submsg =~ s/%s/$mode/;
   $submsg =~ s/%s/$message/;
   $this->post( $subargs->{"usernick"},$submsg) if($subargs->{"usernick"});
- $this->print("adEmote:".Dumper( $this->{"tags"}->{$token})."\n");
+ #$this->print("addEmote:".Dumper( $this->{"tags"}->{$token})."\n");
 
   return 1;
 }#sub addEmote
@@ -885,6 +946,7 @@ sub completeMsg
   my ($this,$msg, $data) = @_;
   #$this->print("called complete msg with $msg\n");
   my $botname = $data->{"heap"}->{"nick"};
+  $botname = "toutou" if(!$botname);
 
 
   #initialize the vars in case they aren't initialized, avoid warnings...
@@ -894,6 +956,7 @@ sub completeMsg
   $data->{"target"} = " " if(!($data->{"target"}));
   $data->{"channel"} = " " if(!($data->{"channel"}));
   $data->{"line"} = " " if(!($data->{"line"}));
+  return $msg if(!$msg);
 
   
   $msg =~ s/%sender/$data->{"usernick"}/g;
@@ -902,7 +965,11 @@ sub completeMsg
   $msg =~ s/%target/$data->{"target"}/g;
   $msg =~ s/%channel/$data->{"channel"}/g;
   $msg =~ s/%rest/$data->{"line"}/g;
-  $msg =~ s/%botname/$botname/g;
+  $msg =~ s/%arg1/$data->{"arg1"}/g;
+  $msg =~ s/%arg2/$data->{"arg2"}/g;
+  $msg =~ s/%arg3/$data->{"arg3"}/g;
+  $msg =~ s/%arg4/$data->{"arg4"}/g;
+  $msg =~ s/%botname/$botname/g if($botname);
   $msg =~ s/%C//g; #mirc style color codes
 
   $msg =~ s/\\n/\n/g;
@@ -936,10 +1003,42 @@ sub completeMsg
   $msg =~ s/\+Ocirc;//g;
   $msg =~ s/\&lt;/</g;
   $msg =~ s/\&gt;/>/g;
+  $msg =~ s/lt;/</g;
+  $msg =~ s/gt;/>/g;
 
+  #until i understand how to convert to iso-latin....
+  $msg =~ s//e/g;
+  $msg =~ s//e/g;
+  $msg =~ s//e/g;
+  $msg =~ s//a/g;
+  $msg =~ s//i/g;
+  $msg =~ s//c/g;
 
   return $msg;
 }#sub completeMsg
+#########################################################
+=pod
+
+=item emoteToXml
+
+transform an emote into a valid xml string
+
+=cut
+
+#########################################################
+sub emoteToXml
+{
+  my($this,$name,$line) = @_;
+  my $entry  = "<emote tag=\"$name\"";
+  $entry .= " active=\"".$line->{"active"}."\"" if($line->{"active"});
+  $entry .= ">\n";
+  $entry .= $this->dumpEmote($line);
+  $entry .= "</emote>\n";
+
+  return $entry;
+}#end emoteToXml
+
+### EOC
 1
 __END__
 
