@@ -84,6 +84,7 @@ sub init
 
   $this->SUPER::init($sysref);
   $this->sysdata($sysref);
+  $sysref->module("pam",$this);
 
   $this->setting("database","DBI:Pg:dbname=zebot") if(!($this->setting("database")));
   #stat a session since we need an instance able to receive and process the DBI Agent callbacks
@@ -134,9 +135,11 @@ This module reacts only to messages send on the channels is active by invoquing
 sub PRIVMSGaction
 {
   my ($this,$splittedline) = @_;
+  $this->print("PAM before active\n");
   return 0 if($this->SUPER::activeForThisChan($splittedline));
+  $this->print("PAM after active\n");
   my $line = $splittedline->{"line"};
-  #print("PAM PRIVMSGaction with $line\n");
+ print("PAM PRIVMSGaction with $line\n");
   my $usernick = $splittedline->{"usernick"};
   my $username = $splittedline->{"username"};
   my $userhost = $splittedline->{"userhost"};
@@ -145,7 +148,7 @@ sub PRIVMSGaction
   $this->{"context"} = $splittedline->{"context"};
   #$this->print( "pam::PRIVMSGaction just set kernel=". $this->{"kernel"} );
 
-  my $msgHand = ($this->sysdata())->{"actors"}->{"messageHandler"};
+  my $msgHand = ($this->sysdata())->module("messageHandler");
   #my $actLang = $msgHand->fetchChannelLang($splittedline->{"channel"});
   my $actLang = $splittedline->{"lang"};
   #$splittedline->{"lang"} = $actLang;
@@ -174,6 +177,8 @@ sub PRIVMSGaction
   $splittedline = \%temp;
   $splittedline->{"line"} = $line;
 
+  $this->print("PAM commans is $command\n");
+
   if($command =~ /passwd/)
   {
     return $this->changePasswd($msgHand,$splittedline);
@@ -255,7 +260,8 @@ sub changePasswd
   {
     #$this->print("wrong number of args....");
     #$this->post( $usernick,"syntax: passwd [#chan] oldpass newpass");
-    $this->post( $usernick,$msgHand->getMesg("PAMPASSHELP",$splittedline));
+    $splittedline->{"line"} = $msgHand->getMesg("PAMPASSHELP",$splittedline);
+	$this->post($usernick,$splittedline);
     return 1;
   }# if(scalar(@authaargs)<3)
   #my $oldpass = $1;
@@ -277,7 +283,8 @@ sub changePasswd
     $this->debug("old pass ok : $oldpass vs ".$this->{"users"}->{$lcnick}->{"passwd"});
     $this->{"users"}->{$lcnick}->{"passwd"} = $oldpass;
     #$this->post( $usernick,'old pass ok');
-    $this->post( $usernick,$msgHand->getMesg("PAMOLDPASSOK",$splittedline));
+    $splittedline->{"line"} = $msgHand->getMesg("PAMOLDPASSOK",$splittedline);
+	$this->post($usernick,$splittedline);
     if($this->{"users"}->{$lcnick})
     {
       my $userid = $this->{"users"}->{$lcnick}->{"id"};
@@ -297,7 +304,8 @@ sub changePasswd
     {
       $this->debug("no pass change to $usernick");
       #$this->post( $usernick,"DB prob, no pass change for $usernick $username\@$userhost: $sql_query");
-      $this->post( $usernick,$msgHand->getMesg("PAMDBPROB",$splittedline)." $usernick $username\@$userhost: ");
+    $splittedline->{"line"} = $msgHand->getMesg("PAMDBPROB",$splittedline)." $usernick $username\@$userhost: ";
+	$this->post($usernick,$splittedline);
     }
 
   }# if($this->{"users"}->{$data->{"nick"}}->{"passwd"})
@@ -305,7 +313,8 @@ sub changePasswd
   {
     $this->debug("wrong credentials: no pass change to $usernick");
     #$this->post( $usernick,"wrong credentials: no pass change for $usernick $username\@$userhost");
-    $this->post( $usernick,$msgHand->getMesg("PAMLOGINERROR",$splittedline)." $usernick $username\@$userhost");
+    $splittedline->{"line"} = $msgHand->getMesg("PAMLOGINERROR",$splittedline)." $usernick $username\@$userhost: ";
+	$this->post($usernick,$splittedline);
   }
   return 1;
 }#sub changePasswd
@@ -337,7 +346,8 @@ sub authenticate
   if(scalar(@authaargs) == 0 )
   {
     #$this->post( $usernick,"syntax: auth [#chan] [user] passwd ");
-    $this->post( $usernick,$msgHand->getMesg("PAMAUTHHELP",$splittedline));
+    $splittedline->{"line"} = $msgHand->getMesg("PAMAUTHHELP",$splittedline);
+    $this->post($usernick,$splittedline);
     return 1;
   }# if(scalar(@authaargs)<3)
 
@@ -362,7 +372,8 @@ sub authenticate
   {
     #check if the pass is valid
     #$this->post( $usernick,"Discordance between your nick and given nick");
-    $this->post( $usernick,$msgHand->getMesg("PAMDIFFNICK",$splittedline));
+    $splittedline->{"line"} = $msgHand->getMesg("PAMDIFFNICK",$splittedline);
+    $this->post($usernick,$splittedline);
     my $lcnick = lc($authuser);
     if($this->{"users"}->{$lcnick} && 
 	$this->{"users"}->{$lcnick}->{"passwd"} eq $itspass)
@@ -390,62 +401,83 @@ sub authenticate
     {
       #$this->print( "pam::PRIVMSGaction user seems valid" );
       #$this->post(  $usernick,"User detected");
-      $this->post(  $usernick,$msgHand->getMesg("PAMUSERFOUND",$splittedline));
+      $splittedline->{"line"} = $msgHand->getMesg("PAMUSERFOUND",$splittedline);
+      $this->post($usernick,$splittedline);
+      #TODO security risk here, this bit should be resetted on user quit,
+      #discon etc.. alas i don't know how to reliabily detect if the user that
+      #authenticated is still there: scenarios, a user takes over the comp
+      #after the legitimate one: user and hostmaks are the same... but we don't
+      #want either to have to authenticate at each discon... argh...
       $this->{"users"}->{$lcnick}->{"auth"} = 1;
+      $this->{"users"}->{$lcnick}->{"lineID"} = $splittedline->{"lineID"}
+      if($splittedline->{"lineID"});
       #$this->post( $usernick,'authentifyed');
-      $this->post( $usernick,$msgHand->getMesg("PAMAUTHOK",$splittedline));
+      $splittedline->{"line"} = $msgHand->getMesg("PAMAUTHOK",$splittedline);
+      $this->post($usernick,$splittedline);
     }# if $this->{"users"}->{$usernick} && 
     else
     {
       #wrong pass,
       #$this->print( "pam::PRIVMSGaction pass not ok" );
       #$this->post( $usernick,"User with wrong credentials");
-      $this->post( $usernick,$msgHand->getMesg("PAMNOAUTH",$splittedline));
+      $splittedline->{"line"} = $msgHand->getMesg("PAMNOAUTH",$splittedline);
+      $this->post($usernick,$splittedline);
       #check for root passwd
       if($itspass eq $this->setting("ownerpass"))
       {
 	#$this->print( "pam::PRIVMSGaction owner!" );
 	#$this->post(  $usernick,"you used the ownerpass!!");
-	$this->post($usernick,$msgHand->getMesg("PAMOWNERPASS",$splittedline));
+	$splittedline->{"line"} = $msgHand->getMesg("PAMOWNERPASS",$splittedline);
+	$this->post($usernick,$splittedline);
 	#set this user as owner
 	if(index($accessRights,"owner") == -1)
 	{
 	  #$this->post(  $usernick,"but your are not yet owner");
-	  $this->post(  $usernick,$msgHand->getMesg("PAMNOTYETOWNER",$splittedline));
+	  $splittedline->{"line"} = $msgHand->getMesg("PAMNOTYETOWNER",$splittedline);
+	  $this->post($usernick,$splittedline);
 	  $this->addToGroup($splittedline,"owner",$usernick);
 	  #$this->post(  $usernick,"new owner $usernick $username\@$userhost");
-	  $this->post(  $usernick,$msgHand->getMesg("PAMNEWOWNER",$splittedline)." $usernick $username\@$userhost");
+	  $splittedline->{"line"} = $msgHand->getMesg("PAMNEWOWNER",$splittedline);
+	  $splittedline->{"line"} .= " $usernick $username\@$userhost";
+	  $this->post($usernick,$splittedline);
 	  $this->{"users"}->{$lcnick}->{"auth"} = 1;
 
 	  my $network = $splittedline->{"network"};
 	  my $channel = $splittedline->{"channel"};
 
-	  $this->{"users"}->{$lcnick}->{"network"}->{$network}->{$channel} = "
-	    " if(!($this->{"users"}->{$lcnick}->{"network"}->{$network}->{$channel}));
+	  $this->{"users"}->{$lcnick}->{"network"}->{$network}->{$channel} = "owner" if(!($this->{"users"}->{$lcnick}->{"network"}->{$network}->{$channel}));
 
 	  $this->{"users"}->{$lcnick}->{"network"}->{$network}->{$channel} .=
 	    ",owner" if(!($this->{"users"}->{$lcnick}->{"network"}->{$network}->{$channel} =~ /owner/));
 
+	  $this->{"users"}->{$lcnick}->{"lineID"} = $splittedline->{"lineID"}
+	  if($splittedline->{"lineID"});
 	}#if(index($accessRights,"owner") == -1)
       }
       #$this->print( "pam::PRIVMSGaction exiting!" );
       #$this->post(  $usernick,"userrecord for $usernick =
-      #".Dumper($this->{"users"}->{$lcnick}));
+	  #".Dumper($this->{"users"}->{$lcnick}));
     }
   }#if($this->{"users"}->{$usernick})
   else
   {
-      #inexistent user...
-	#$this->print( "pam::PRIVMSGaction no userk" );
-	#$this->post(  $usernick,"inexistent user");
-	#$this->post(  $usernick,Dumper($this->{"users"}));
-	$this->post(  $usernick,$msgHand->getMesg("PAM you have to register prior to authenticate",$splittedline));
+    #inexistent user...
+    #$this->print( "pam::PRIVMSGaction no userk" );
+    #$this->post(  $usernick,"inexistent user");
+    #$this->post(  $usernick,Dumper($this->{"users"}));
+    #my $kernel = $splittedline->{"kernel"};
+    #delete($splittedline->{"kernel"});
+    #print("pam error: ".Dumper($splittedline)."\n");
+    $splittedline->{"line"} = $msgHand->getMesg("PAM you have to register prior
+	to authenticate",$splittedline);
+    $this->post($usernick,$splittedline);
   }
   #$this->print( "pam::PRIVMSGaction exiting!" );
   #$this->post(  $usernick,"userrecord for $usernick = ".Dumper($this->{"users"}->{$usernick}));
   #$this->post(  $usernick,"end auth for $usernick $username\@$userhost");
   #$this->post(  $usernick,$msgHand->getMesg("PAMENDAUTH",$splittedline)." $usernick $username\@$userhost");
-  $this->post(  $usernick,$msgHand->getMesg("PAMENDAUTH",$splittedline));
+  $splittedline->{"line"} = $msgHand->getMesg("PAMENDAUTH",$splittedline);
+  $this->post($usernick,$splittedline);
   return 1;
 }#sub authenticate
 ######################################################################
@@ -477,7 +509,8 @@ sub register
   if(scalar(@authaargs) == 0 )
   {
     #$this->post(  $usernick,"syntax: register [#chan] passwd [email]");
-    $this->post($usernick,$msgHand->getMesg("PAMREGISTERHELP",$splittedline));
+    $splittedline->{"line"} = $msgHand->getMesg("PAMREGISTERHELP",$splittedline);
+	$this->post($usernick,$splittedline);
     return 1;
   }# if(scalar(@authaargs) == 0 )
   #$this->print( "pam::PRIVMSGaction auth kernel=". $this->{"kernel"} );
@@ -498,19 +531,20 @@ sub register
   my $lcnick = lc($usernick);
   if($this->{"users"}->{$lcnick})
   {
-    print("Dump the whole... ".Dumper($this->{"users"}->{$lcnick})."\n");
+    #print("Dump the whole... ".Dumper($this->{"users"}->{$lcnick})."\n");
     my $deb = "CMP ".$this->{"users"}->{$lcnick}->{"name"};
     $deb .= " vs $username ";
     $deb .= $this->{"users"}->{$lcnick}->{"host"};
     $deb .= " vs $userhost ";
     $deb .= $this->{"users"}->{$lcnick}->{"passwd"};
     $deb .= " vs $itspass ";
-    print("$deb\n");
+    #print("$deb\n");
 
     if( $this->{"users"}->{$lcnick}->{"passwd"} ne $itspass)
       {
       #$this->post(  $usernick,"requested nick allready exists");
-      $this->post(  $usernick,$msgHand->getMesg("PAMNICKTAKEN",$splittedline));
+    $splittedline->{"line"} = $msgHand->getMesg("PAMNICKTAKEN",$splittedline);
+	$this->post($usernick,$splittedline);
       return;
       }
     if( $this->{"users"}->{$lcnick}->{"host"} ne $userhost)
@@ -549,7 +583,8 @@ sub register
 
     $this->addUser($splittedline);
     #$this->post(  $usernick,"user $usernick registered with $itspass");
-    $this->post(  $usernick,$msgHand->getMesg("PAMREGISTEROK",$splittedline));
+    $splittedline->{"line"} = $msgHand->getMesg("PAMREGISTEROK",$splittedline);
+	$this->post($usernick,$splittedline);
     #TODO add user to user basis
     #check for root passwd
     if($itspass eq $this->setting("ownerpass"))
@@ -557,20 +592,25 @@ sub register
       $this->{"users"}->{$lcnick}->{"network"}->{$network}->{$channel} .=
 	",owner" if(!($this->{"users"}->{$lcnick}->{"network"}->{$network}->{$channel} =~ /owner/));
       #$this->post(  $usernick,"you used the ownerpass!!");
-      $this->post(  $usernick,$msgHand->getMesg("PAMOWNERPASS",$splittedline));
+    $splittedline->{"line"} = $msgHand->getMesg("PAMOWNERPASS",$splittedline);
+	$this->post($usernick,$splittedline);
       #set this user as owner
       if(index($accessRights,"owner") == -1)
       {
 	#$this->post(  $usernick,"but your are not yet owner");
-	$this->post(  $usernick,$msgHand->getMesg("PAMNOTYETOWNER",$splittedline));
+    $splittedline->{"line"} = $msgHand->getMesg("PAMNOTYETOWNER",$splittedline);
+	$this->post($usernick,$splittedline);
 	$this->addToGroup($splittedline,"owner",$usernick);
 	#$this->post(  $usernick,"new owner $usernick $username\@$userhost");
-	$this->post(  $usernick,$msgHand->getMesg("PAMNEWOWNER",$splittedline)." $usernick $username\@$userhost");
+    $splittedline->{"line"} = $msgHand->getMesg("PAMNEWOWNER",$splittedline);
+	$this->post($usernick,$splittedline);
       }#if(index($accessRights,"owner") == -1)
     }# if($itspass eq $this->setting("ownerpass"))
   
   #$this->post(  $usernick,"end register for $usernick $username\@$userhost");
-  $this->post(  $usernick,$msgHand->getMesg("PAMENDREGISTER",$splittedline)." $usernick $username\@$userhost");
+    $splittedline->{"line"} = $msgHand->getMesg("PAMENDREGISTER",$splittedline);
+    $splittedline->{"line"} .= " $usernick $username\@$userhost";
+	$this->post($usernick,$splittedline);
   #$this->post(  $usernick,"act list of users: ".Dumper($this->{"users"}));
   return 1;
 }#sub register
@@ -598,7 +638,8 @@ sub unregister
   if(!$line)
   {
     #$this->post(  $usernick,"syntax: register [#chan] passwd [email]");
-    $this->post($usernick,$msgHand->getMesg("PAMYou have to supply the password",$splittedline));
+    $splittedline->{"line"} = $msgHand->getMesg("PAMYou have to supply the password",$splittedline);
+	$this->post($usernick,$splittedline);
     return 1;
   }# if(scalar(@authaargs) == 0 )
   my $lcnick = lc($usernick);
@@ -606,15 +647,19 @@ sub unregister
   {
     delete $this->{"users"}->{$lcnick};
     $this->delUser($splittedline);
-    $this->post(  $usernick,$msgHand->getMesg("PAMSuccessfully unregistered user $usernick",$splittedline));
+    $splittedline->{"line"} = $msgHand->getMesg("PAMSuccessfully unregistered user",$splittedline)." $usernick";
+	$this->post($usernick,$splittedline);
   }# if($this->{"users"}->{$authuser})
   else
   {
-    $this->post(  $usernick,$msgHand->getMesg("PAM This user wasn't registered",$splittedline));
+    $splittedline->{"line"} = $msgHand->getMesg("PAM This user wasn't registered",$splittedline);
+	$this->post($usernick,$splittedline);
     #check for root passwd
   }#else
   #$this->post(  $usernick,"end register for $usernick $username\@$userhost");
-  $this->post(  $usernick,$msgHand->getMesg("PAMENDREGISTER",$splittedline)." $usernick $username\@$userhost");
+    $splittedline->{"line"} = $msgHand->getMesg("PAMENDREGISTER",$splittedline);
+    $splittedline->{"line"} .= " $usernick $username\@$userhost";
+	$this->post($usernick,$splittedline);
   #$this->post(  $usernick,"act list of users: ".Dumper($this->{"users"}));
   return 1;
 }#sub unregister
@@ -666,6 +711,8 @@ sub diplayUser
   $listing  = $msgHand->getMesg("PAMCACHEDUSERS",$splittedline)."\n$listing";
   $listing .= "\n".$msgHand->getMesg("PAMENDUSERLIST",$splittedline)."\n";
   $this->post( $usernick,$listing);
+    $splittedline->{"line"} .= $listing;
+	$this->post($usernick,$splittedline);
 
   return 1;
 }#sub diplayUser
@@ -691,11 +738,13 @@ sub addUserToGroup
   if(!$usertoadd || !$usergroup)
   {
     #$this->post(  $usernick,"you forgot the args user grp");
-    $this->post(  $usernick,$msgHand->getMesg("PAMADDUSERHELP",$splittedline));
+    $splittedline->{"line"} = $msgHand->getMesg("PAMADDUSERHELP",$splittedline);
+    $this->post($usernick,$splittedline);
     return 1;
   }# if(!$usertoadd)
   #$this->post(  $usernick,"asdduser: $usertoadd to grp $usergroup");
-  $this->post(  $usernick,$msgHand->getMesg("PAMADDUSERDEBUG",$splittedline));
+    $splittedline->{"line"} = $msgHand->getMesg("PAMADDUSERDEBUG",$splittedline);
+    $this->post($usernick,$splittedline);
 
 
   #$this->print( "pam: debug '$botname': case is adduser !");
@@ -703,7 +752,7 @@ sub addUserToGroup
   #$this->print( "m1: search for $usernick");
   return 1 if($usergroup eq "owner" && index($accessRights,"owner")==-1);
 
-  print("casting PAM addToGroup \n");
+  #print("casting PAM addToGroup \n");
   $this->addToGroup($splittedline,$usergroup,$usertoadd);
 
   return 1;
@@ -769,7 +818,7 @@ sub destroyTables
 
 =item addAGroup
 
-register a user
+create a new group
 
 =cut
 
@@ -788,15 +837,15 @@ sub addAGroup
   my $queries = [];
   my $sql_query = 'INSERT INTO "groups" ( "name") VALUES (\''.$newgroup.'\')';
 
-      push(@$queries,$sql_query);
+  push(@$queries,$sql_query);
 
-      $splittedline->{"query"} = $queries;
-      $splittedline->{"type"} = "none";
+  $splittedline->{"query"} = $queries;
+  $splittedline->{"type"} = "none";
 
-      $splittedline->{"session"} = $this->session();
-      $splittedline->{"event"} = "addedAgroup";
-      $kernel->post( $this->{"DBslave"}, 'batch',$splittedline);
-      return 1;
+  $splittedline->{"session"} = $this->session();
+  $splittedline->{"event"} = "addedAgroup";
+  $kernel->post( $this->{"DBslave"}, 'batch',$splittedline);
+  return 1;
 }#sub addAGroup
 ######################################################################
 =pod
@@ -819,12 +868,12 @@ sub showAllGroups
 
   my $sql_query = 'SELECT * FROM "groups"';
 
-      $splittedline->{"query"} = $sql_query;
-      $splittedline->{"type"} = "hash";
+  $splittedline->{"query"} = $sql_query;
+  $splittedline->{"type"} = "hash";
 
-      $splittedline->{"session"} = $this->session();
-      $splittedline->{"event"} = "groupsDisplayed";
-      $kernel->post( $this->{"DBslave"}, 'doQuery',$splittedline);
+  $splittedline->{"session"} = $this->session();
+  $splittedline->{"event"} = "groupsDisplayed";
+  $kernel->post( $this->{"DBslave"}, 'doQuery',$splittedline);
   return 1;
 }#sub showAllGroups
 ######################################################################
@@ -889,10 +938,10 @@ sub setNicksLogged
   my $channel = $splittedline->{"channel"};
 
 
-  my $line = {};
-  $line->{"kernel"} = $kernel;
-  $line->{"context"} = $context;
-  $line->{"channel"} = $channel;
+  my $fakeLine = {};
+  $fakeLine->{"kernel"} = $kernel;
+  $fakeLine->{"context"} = $context;
+  $fakeLine->{"channel"} = $channel;
   foreach my $username (@$names)
   {
     $username =~ s/~//g;
@@ -907,19 +956,20 @@ sub setNicksLogged
     if($username && ($username ne $botname))
     {
       #$this->print "updating: $username";
-      $line->{"usernick"} = $username;
-      $line->{"username"} = $username;
-      $line->{"userhost"} = $username;
-      my $accessRights = findOperator($this,$line);
+      $fakeLine->{"usernick"} = $username;
+      $fakeLine->{"username"} = $username;
+      $fakeLine->{"userhost"} = $username;
+      my $accessRights = findOperator($this,$fakeLine);
       #if(index($accessRights,"owner") != -1 || index($accessRights,"oper") != -1)
-      if($this->isOwnerOrOper($line))
+      if($this->isOwnerOrOper($fakeLine))
       {
-	my $msgHand = ($this->sysdata())->{"actors"}->{"messageHandler"};
+	my $msgHand = ($this->sysdata())->module("messageHandler");
 	#my $actLang = $msgHand->fetchChannelLang($channel);
 	my $actLang = $splittedline->{"lang"};
 	#$splittedline->{"lang"} = $actLang;
-	$this->post($username,$botname.' '.$msgHand->getMesg("PAMREAUTH",$splittedline));
-
+	$splittedline->{"line"} = $botname.' ';
+	$splittedline->{"line"} .= $msgHand->getMesg("PAMREAUTH",$splittedline);
+	$this->post($username,$splittedline);
       }# if(!(index($accessRights,"owner") != -1 || index($accessRights,"oper") != -1))
     }# if($usernick ne $botname")) 
   }#foreach my $username (@$names)
@@ -1014,45 +1064,12 @@ issue the help for this module
 ######################################################################
 sub help
 {
-  my $this = shift;
-  my $splittedline = shift;
-  my $botname = $splittedline->{"heap"}->{"nick"};
-  #$this->print "m6: search for $usernick";
-  #$this->print "ref = '$splittedline'";
-  my $usernick = $splittedline->{"usernick"};
-  my $username = $splittedline->{"username"};
-  my $userhost = $splittedline->{"userhost"};
-  my $kernel = $splittedline->{"kernel"};
-  $this->{"kernel"} = $splittedline->{"kernel"};
-  $this->{"context"} = $splittedline->{"context"};
-  my $msgHand = ($this->sysdata())->{"actors"}->{"messageHandler"};
+  my ($this, $splittedline) = @_;
+  my $msgHand = ($this->sysdata())->module("messageHandler");
 
 
-  #$this->print "help, req by $usernick $username\@$userhost";
-  #$this->print "full-line was ".$splittedline->{"fulline"};
-  #$this->print "-- help FO with ".Dumper($splittedline);
-  my $list = findOperator($this,$splittedline);
   my $helpmsg = $msgHand->getMesg("PAMHELP",$splittedline);
   return $helpmsg;
-  #my $helpmsg = "PAM-help:\n";
-  #$helpmsg   .= "   set password: passwd <old passwd> <new passwd>\n";
-  #$helpmsg   .= "   authenticate: auth <nick> <passwd>\n";
-  #if($list ne "")
-  #{
-    #  $helpmsg   .= "   add user: adduser nick [oper|teki|ally|user]\n";
-    #  $helpmsg   .= "   add group: addgroup name\n";
-    #  $helpmsg   .= "   remove user: deluser nick [oper|teki|ally|user]\n";
-    #  $helpmsg   .= "   display user: dispuser [groupname]\n";
-    #  $helpmsg   .= "   display groups: listgroups\n";
-    #  return $helpmsg;
-    #}#if($list ne "")
-  #else 
-  #{ 
-    #  $helpmsg .= "user $usernick, $username\@$userhost is not ";
-    #  $helpmsg .= "Operator, help not available!\n";
-    #  #$helpmsg .= "result query gave: $list";
-    #  return $helpmsg;
-    #}
 }#sub help
 ######################################################################
 =pod
@@ -1137,7 +1154,9 @@ sub getLogged
 
 =item findOperator 
 
-return the rights of this user, rights owner and oper are normally only given if the user is authentificated, possible to bypass by giving a second argument set to true
+return the rights of this user, rights owner and oper are normally only given
+if the user is authentificated, possible to bypass by giving a second argument
+set to true
 
 =cut
 
@@ -1190,6 +1209,8 @@ sub findOperator
 	#print("noauth ");
 	$rights =~ s/owner|oper//gi if($rights);
       }# if(!($noauth || $this->{"users"}->{$nick}->{"auth"}))
+      #internal modus, push the session id onto the stack
+      $rights .= " ".$this->{"users"}->{$nick}->{"lineID"} if($this->{"users"}->{$nick}->{"lineID"});
     }# if($this->{"users"}->{$nick})
     else
     {
@@ -1242,7 +1263,7 @@ return the version of this module
 ######################################################################
 sub version
 {
-  return '$Revision: 1.33 $';
+  return '$Revision: 1.38 $';
 }#sub isa
 ######################################################################
 =pod
@@ -1335,7 +1356,7 @@ sub addToGroup
   $this->{"kernel"} = $line->{"kernel"};
   $this->{"context"} = $line->{"context"};
 
-  my $msgHand = ($this->sysdata())->{"actors"}->{"messageHandler"};
+  my $msgHand = ($this->sysdata())->module("messageHandler");
   my $actLang = $line->{"lang"};
 
   print("PAM::addToGroup $userid to $usergroup\n");
@@ -1366,20 +1387,30 @@ sub addToGroup
 
       $line->{"session"} = $this->session();
       $line->{"event"} = "usergroupMatch";
-      print("posting $sql_query\n");
+      $this->print("pam::addToGroup posting $sql_query\n");
       $kernel->post( $this->{"DBslave"}, 'doQuery',$line);
 
+      my $network = $line->{"network"};
+      my $channel = $line->{"channel"};
+      my $userRecord = $this->{"users"}->{$userid}->{"network"}->{$network};
+
+      $userRecord->{$channel} = $usergroup if(!($userRecord->{$channel}));
+
+      $userRecord->{$channel} .= ",$usergroup"
+	if(!($userRecord->{$channel} =~ /$usergroup/));
     }# if($groupid)
     else
     {
       #$this->post( $usernick,'no such group');
-      $this->post( $usernick,$msgHand->getMesg("PAMGROUPUNKNOWN",$line));
+    $line->{"line"} = $msgHand->getMesg("PAMGROUPUNKNOWN",$line);
+    $this->post($usernick,$line);
     }#else
   }# if($this->{"users"}->{$userid})
   else
   {
-    print("register the user first.... !\n");
-    $this->post( $usernick,$msgHand->getMesg("PAM you have to register the user first",$line));
+    #print("register the user first.... !\n");
+    $line->{"line"} = $msgHand->getMesg("PAM you have to register the user first",$line);
+    $this->post($usernick,$line);
   }# else
 }#sub addToGroup
 ######################################################################
@@ -1408,22 +1439,21 @@ sub addUser
   my $lcnick = lc($usernick);
   my   $sql_query = 'INSERT INTO "users" (nick,name,host,access,passwd) VALUES';
   $sql_query .= "('$lcnick','$username','$userhost',CURRENT_DATE,'$passwd')";
-  print("PAM ADD USER: $sql_query\n");
   #$this->print ("### inserting new user:$sql_query");
   push(@queries,$sql_query);
   $sql_query = 'INSERT INTO group2user ("user","group",channel,network) VALUES';
   $sql_query .= "( (SELECT id from \"users\" where nick='$lcnick'),";
   $sql_query .= "  (SELECT id from \"groups\" where name='user'),";
   $sql_query .= "  '$channel','$network')";
-  print("PAM ADD USER: $sql_query\n");
   push(@queries,$sql_query);
 
-      $splittedline->{"query"} = \@queries;
-      $splittedline->{"type"} = "none";
+  $splittedline->{"query"} = \@queries;
+  $splittedline->{"type"} = "none";
 
-      $splittedline->{"session"} = $this->session();
-      $splittedline->{"event"} = "ignoreReturn";
-      $kernel->post( $this->{"DBslave"}, 'batch',$splittedline);
+  $splittedline->{"session"} = $this->session();
+  $splittedline->{"event"} = "ignoreReturn";
+  #$this->print("posting to ".$this->{"DBslave"}." data ".Dumper(@queries)."\n");
+  $kernel->post( $this->{"DBslave"}, 'batch',$splittedline);
   #TODO send ignoreReturn event
   #$this->print "### adduser prob:".$sth->errstr if(!$sth);
 }#sub addUser
@@ -1589,13 +1619,13 @@ sub join_success
   print("PAM fetchig users for net: ".$args->{"heap"}->{"network"}."\n");
   #$this->print("JOINSUCCES!!!! casting events for DB handling !");
 
-      $args->{"values"} = [$args->{"channel"},$args->{"heap"}->{"network"}];
-      $args->{"query"} = $sql_query;
-      $args->{"type"} = "hash";
+  $args->{"values"} = [$args->{"channel"},$args->{"heap"}->{"network"}];
+  $args->{"query"} = $sql_query;
+  $args->{"type"} = "hash";
 
-      $args->{"session"} = $this->session();
-      $args->{"event"} = "fetchedUsers";
-      $args->{"kernel"}->post( $this->{"DBslave"}, 'doQuery',$args);
+  $args->{"session"} = $this->session();
+  $args->{"event"} = "fetchedUsers";
+  $args->{"kernel"}->post( $this->{"DBslave"}, 'doQuery',$args);
   return 0;
 }#sub join_success
 ##############################################################################
@@ -1725,16 +1755,18 @@ sub passChanged
 {
   my ($this,$kernel, $heap, $result) = @_[OBJECT, KERNEL, HEAP, ARG0];
   my $errmsg = "";
-  my $msgHand = ($this->sysdata())->{"actors"}->{"messageHandler"};
+  my $msgHand = ($this->sysdata())->module("messageHandler");
   my $usernick = $result->{"nick"};
 
   if($result->{"status"} eq "failure")
   {
     $this->print("something went wrong with PAM::changepass ".$result->{"values"}."\n");
-  $this->post(  $usernick,$msgHand->getMesg("PAM failed to change the passwd in the database",$result));
+    $result->{"line"} = $msgHand->getMesg("PAM failed to change the passwd in the database",$result);
+    $this->post($usernick,$result);
   return;
   }# if($result->{"status"} eq "failure")
-  $this->post(  $usernick,$msgHand->getMesg("PAMNEWPASSOK",$result));
+    $result->{"line"} = $msgHand->getMesg("PAMNEWPASSOK",$result);
+    $this->post($usernick,$result);
   $this->{"users"}->{lc($usernick)}->{"passwd"} = $result->{"newpass"};
   $this->debug("change pass ok");
 }#sub passChanged
@@ -1752,35 +1784,39 @@ sub groupsDisplayed
 {
   my ($this,$kernel, $heap, $result) = @_[OBJECT, KERNEL, HEAP, ARG0];
   my $errmsg = "";
-  my $msgHand = ($this->sysdata())->{"actors"}->{"messageHandler"};
+  my $msgHand = ($this->sysdata())->module("messageHandler");
   my $usernick = $result->{"usernick"};
 
   if($result->{"status"} eq "failure")
   {
-  $this->post(  $usernick,$msgHand->getMesg("PAM database error for diplsay groups",$result));
-  return;
+    $result->{"line"} = $msgHand->getMesg("PAM database error for display groups",$result);
+    $this->post($usernick,$result);
+    return;
   }# if($result->{"status"} eq "failure")
+  $result->{"values"} = [] if(!($result->{"values"}));
   my $numentries =  scalar(@{$result->{"values"}});
   if($numentries)
   {
     #$this->post( $usernick,'List of available groups');
     print("posting to $usernick ".$msgHand->getMesg("PAMLISTGROUPS",$result)." altnick:".$result->{"usernick"}."\n");
-    $this->post( $usernick,$msgHand->getMesg("PAMLISTGROUPS",$result));
-    $this->post( $usernick,'num='.$numentries);
-    #$this->post( $usernick,'id  name');
-    $this->post( $usernick,$msgHand->getMesg("PAMLISTGROUPSHEAD",$result));
+    $result->{"line"} = $msgHand->getMesg("PAMLISTGROUPS",$result)."\n";
+    $result->{"line"} .= 'num='.$numentries."\n";
+    $result->{"line"} .= $msgHand->getMesg("PAMLISTGROUPSHEAD",$result);
     # Read the matching records and$this->print them out          
     foreach my $data (@{$result->{"values"}})
     {
-      $this->post(  $usernick,''.$data->{"id"}." ".$data->{"name"});
+    $result->{"line"} .= $data->{"id"}." ".$data->{"name"}."\n";
     }
+    $this->post($usernick,$result);
   }#if( $sth->rows != 0)
   else
   {
-      print("posting to $usernick no entries in the DB response... \n");
-      $this->post(  $usernick,'no groups to display');
+    print("posting to $usernick no entries in the DB response... \n");
+    $result->{"line"} = $msgHand->getMesg("PAM no groups to display",$result);
+    $this->post($usernick,$result);
   }
-  $this->post($usernick,''.$msgHand->getMesg("PAMLISTGROUPOK",$result));
+    $result->{"line"} = $msgHand->getMesg("PAMLISTGROUPOK",$result);
+    $this->post($usernick,$result);
 }#sub groupsDisplayed
 ######################################################################
 =pod
@@ -1818,7 +1854,7 @@ sub usergroupMatch
   my $errmsg = "";
   my $network =  $result->{"network"};
   my $channel =  $result->{"channel"};
-  my $msgHand = ($this->sysdata())->{"actors"}->{"messageHandler"};
+  my $msgHand = ($this->sysdata())->module("messageHandler");
   my $usernick = $result->{"nick"};
   my $usergroup = $result->{"group"};
   my $userid = $result->{"userid"};
@@ -1828,6 +1864,7 @@ sub usergroupMatch
   if($result->{"status"} eq "failure")
   {
     $this->print("something went wrong with PAM::addToGroup: ".$result->{"values"}."\n");
+    $this->updateGroupSubscription($kernel,$result);
   }# if($result->{"status"} eq "failure")
   else
   {
@@ -1837,36 +1874,13 @@ sub usergroupMatch
     if($num >0)
     {
       print("PAM::usergroupMatch allready in group!\n");
-      $this->post($usernick,$msgHand->getMesg("PAMALLREADYINGROUP",$result));
+    $result->{"line"} = $msgHand->getMesg("PAMALLREADYINGROUP",$result);
+    $this->post($usernick,$result);
     }# if($num >0)
     else
     {
-      print("PAM::usergroupMatch adding to group!\n");
-      $this->post( $usernick,$msgHand->getMesg("PAMADDTOGROUP",$result));
-      my $lcnick = lc($userid);
-      if($this->{"users"}->{$lcnick}->{"network"}->{$network}->{$channel})
-      {
-	$this->{"users"}->{$lcnick}->{"network"}->{$network}->{$channel} .= ",".$usergroup;
-      }# if($this->{"users"}->{$data->{"nick"}}->{"channels"})
-      else
-      {
-	$this->{"users"}->{$lcnick}->{"network"}->{$network}->{$channel} = $usergroup;
-      }#else
-      print("PAM::usergroupMatch user updated to ".Dumper($this->{"users"}->{$userid})."!\n");
-
-      my $sql_query = 'INSERT INTO group2user ("user","group",channel,network)';
-      $sql_query .= ' VALUES (';
-	  $sql_query .=
-	  "'".$this->{"users"}->{$lcnick}->{"id"}."','$groupid','$channel','$network')";
-
-	  print("PAM::usergroupMatch  issuing : $sql_query\n");
-      $result->{"query"} = $sql_query;
-      $result->{"type"} = "count";
-
-      $result->{"session"} = $this->session();
-      $result->{"event"} = "usertogroupInserted";
-      $kernel->post( $this->{"DBslave"}, 'doQuery',$result);
-    }#else
+      $this->updateGroupSubscription($kernel,$result);
+   }#else
   }# else
 }#sub usergroupMatch
 ###########################################result###############
@@ -1883,16 +1897,18 @@ sub usertogroupInserted
 {
   my ($this,$kernel, $heap, $result) = @_[OBJECT, KERNEL, HEAP, ARG0];
   my $errmsg = "";
-  my $msgHand = ($this->sysdata())->{"actors"}->{"messageHandler"};
+  my $msgHand = ($this->sysdata())->module("messageHandler");
   my $usernick = $result->{"nick"};
 
   if($result->{"status"} eq "failure")
   {
     $this->print("something went wrong with PAM::addToGroup2 ".$result->{"values"}."\n");
-  $this->post(  $usernick,$msgHand->getMesg("PAM database error for insert user into group ",$result));
+    $result->{"line"} = $msgHand->getMesg("PAM database error for insert user into group ",$result);
+    $this->post($usernick,$result);
   return;
   }# if($result->{"status"} eq "failure")
-  $this->post( $usernick,$msgHand->getMesg("PAMADD2GROUPOK",$result));
+    $result->{"line"} = $msgHand->getMesg("PAMADD2GROUPOK",$result);
+    $this->post($usernick,$result);
 }#sub usertogroupInserted
 ######################################################################
 =pod
@@ -1995,7 +2011,7 @@ sub addedAgroup
 {
   my ($this,$kernel, $heap, $result) = @_[OBJECT, KERNEL, HEAP, ARG0];
   my $errmsg = "";
-  my $msgHand = ($this->sysdata())->{"actors"}->{"messageHandler"};
+  my $msgHand = ($this->sysdata())->module("messageHandler");
   my $usernick = $result->{"nick"};
 
   if($result->{"status"} eq "failure")
@@ -2004,7 +2020,8 @@ sub addedAgroup
   }# if($result->{"status"} eq "failure")
   else
   {
-    $this->post($usernick," ".$msgHand->getMesg("PAMADDGROUP",$result));
+    $result->{"line"} = $msgHand->getMesg("PAMADDGROUP",$result);
+    $this->post($usernick,$result);
   }
 }#sub addedAgroup
 ######################################################################
@@ -2067,7 +2084,7 @@ sub fetchedUsers
     my $group = $data->{"group"};
     if(!($this->{"users"}->{$lcnick}))
     {
-      $this->print("new entry");
+      #$this->print("new entry");
 
       $this->{"users"}->{$lcnick} = {
 	"id" => $data->{"id"},
@@ -2097,6 +2114,88 @@ sub fetchedUsers
 
   #$this->print("PAM DB fetch of users complete:: ".Dumper($this->{"users"}));
 }# sub fetchedUsers
+#########################################################
+=pod
+
+=item updateGroupSubscription
+
+update the database with the subscription of a user to a group
+
+=cut
+
+#########################################################
+sub updateGroupSubscription
+{
+  my ($this,$kernel,$result) = @_;
+  
+  my $network =  $result->{"network"};
+  my $channel =  $result->{"channel"};
+  my $msgHand = ($this->sysdata())->module("messageHandler");
+  my $usernick = $result->{"nick"};
+  my $usergroup = $result->{"group"};
+  my $userid = $result->{"userid"};
+  my $groupid = $result->{"groupid"};
+
+  print("PAM::usergroupMatch adding to group!\n");
+    $result->{"line"} = $msgHand->getMesg("PAMADDTOGROUP",$result);
+    $this->post($usernick,$result);
+  my $lcnick = lc($userid);
+  if($this->{"users"}->{$lcnick}->{"network"}->{$network}->{$channel})
+  {
+    $this->{"users"}->{$lcnick}->{"network"}->{$network}->{$channel} .= ",".$usergroup;
+  }# if($this->{"users"}->{$data->{"nick"}}->{"channels"})
+  else
+  {
+    $this->{"users"}->{$lcnick}->{"network"}->{$network}->{$channel} = $usergroup;
+  }#else
+  print("PAM::usergroupMatch user updated to ".Dumper($this->{"users"}->{$userid})."!\n");
+
+  my $sql_query = 'INSERT INTO group2user ("user","group",channel,network)';
+  $sql_query .= ' VALUES (';
+  if($this->{"users"}->{$lcnick}->{"id"})
+  {
+    $sql_query .= "'".$this->{"users"}->{$lcnick}->{"id"}."',";
+  }# if($this->{"users"}->{$lcnick}->{"id"})
+  else
+  {
+    $sql_query .= "(SELECT id FROM \"users\" WHERE nick='$lcnick'),";
+  }# else
+  $sql_query .= "'$groupid','$channel','$network')";
+
+  $this->print("PAM::updateGroupSubscription  issuing : $sql_query\n");
+  $result->{"query"} = $sql_query;
+  $result->{"type"} = "count";
+
+  $result->{"session"} = $this->session();
+  $result->{"event"} = "usertogroupInserted";
+  $kernel->post( $this->{"DBslave"}, 'doQuery',$result);
+
+  return;
+}#end updateGroupSubscription
+######################################################################
+=pod
+
+=item countRegisteredUsers 
+
+give out how many registered users there are for a given channel
+
+=cut
+
+######################################################################
+sub countRegisteredUsers 
+{
+  my ($this,$network,$channel) = @_;
+
+  my $count = 0;
+  foreach my $nick (keys(%{$this->{"users"}}))
+  {
+   my $user = $this->{"users"}->{$nick}; 
+   $count++ if($user->{"network"}->{$network}->{$channel});
+  }# foreach $user (keys(%{$this->{"users"}}))
+
+  return $count;
+}# sub countRegisteredUsers 
+### EOC
 1
 __END__
 
