@@ -125,7 +125,7 @@ sub PRIVMSGaction
   $this->{"context"} = $splittedline->{"context"};
   #$this->print( "pam_pg::PRIVMSGaction just set kernel=". $this->{"kernel"}."\n" );
 
-  my $msgHand = ($this->sysdata())->{"actors"}->{"messageHandler"};
+  my $msgHand = ($this->sysdata())->module("messageHandler");
   #my $actLang = $msgHand->fetchChannelLang($splittedline->{"channel"});
   #$splittedline->{"lang"} = $actLang;
   my $actLang = $splittedline->{"lang"};
@@ -147,7 +147,7 @@ sub PRIVMSGaction
     # $this->print("case is passwd\n");
     # $this->print("m4: search for $usernick\n");
     my @authaargs = split(/ /,$1);
-  my $msgHand = ($this->sysdata())->{"actors"}->{"messageHandler"};
+  my $msgHand = ($this->sysdata())->module("messageHandler");
   
     if(scalar(@authaargs) != 2 )
     {
@@ -536,13 +536,13 @@ sub setNicksLogged
     #shift @hostparts;
     #$userhost = join('.',@hostparts);
     #bail out if its the same bot joinning
-  my $botname = $splittedline->{"heap"}->{"nick"};
+    my $botname = $splittedline->{"heap"}->{"nick"};
     if($username ne $botname) 
     {
       # $this->print("updating: $username\n");
-  my $msgHand = ($this->sysdata())->{"actors"}->{"messageHandler"};
-  my $actLang = "";
-	$this->post($username,$botname.' '.$msgHand->getMesg("PAMREAUTH",$splittedline));
+      my $msgHand = ($this->sysdata())->module("messageHandler");
+      my $actLang = "";
+      $this->post($username,$botname.' '.$msgHand->getMesg("PAMREAUTH",$splittedline));
     }# if$usernick ne $botname 
   }#foreach my $username (@$names)
   return 0;
@@ -916,7 +916,7 @@ return the version of this module
 ######################################################################
 sub version
 {
-  return '$Revision: 1.11 $';
+  return '$Revision: 1.12 $';
 }#sub isa
 
 ######################################################################
