@@ -1,9 +1,9 @@
 #!/usr/bin/perl -w
 use zebot;
 
-sub POE::Kernel::ASSERT_DEFAULT () { 1 }
+#sub POE::Kernel::ASSERT_DEFAULT () { 1 }
 sub POE::Kernel::ASSERT_PONY () { 1 }
-sub POE::Kernel::ASSERT_EVENTS () { 1 }
+#sub POE::Kernel::ASSERT_EVENTS () { 1 }
 sub POE::Kernel::ASSERT_STATES () { 1 }
 
 use POE::Kernel;
