@@ -474,7 +474,8 @@ sub channel_join_topic
   my ($this,$subargs) = @_[OBJECT,ARG0];
   $subargs->{"kernel"} = $_[KERNEL];
   #print "detected topic: ".$subargs->{server}." for ".$subargs->{args}."\n";
-  print "detected topic: ".Dumper($subargs)."\n";
+  #print "detected topic: ".Dumper($subargs)."\n";
+
   
   #trying to fetch the list of people
   return 0;
@@ -512,23 +513,33 @@ sub channel_join_success
 #########################################################
 =pod
 
-=item giveModules
+=item module
 
 issue the list of installed modules
+ir if an argument was given return the asked module
 
 =cut
 
 #########################################################
-sub giveModules
+sub module
 {
-  my ($this) = @_;
+  my ($this,$arg,$module) = @_;
   my $actorRef = $this->{"actors"};
-  foreach my $i (keys(%$actorRef))
+  if($arg)
   {
-    print("debMainTest: $i ->".$actorRef->{$i}."\n");
-  }#foreach $i (keys(%$this->{"actors"}))
+    if($module)
+    {
+      $actorRef->{$arg} = $module; 
+    }# if($module)
+   return($actorRef->{$arg}); 
+  }# if($arg)
+
+  #foreach my $i (keys(%$actorRef))
+  #{
+  #  print("debMainTest: $i ->".$actorRef->{$i}."\n");
+  #}#foreach $i (keys(%$this->{"actors"}))
   return $actorRef;
-}
+}# sub module
 
 #########################################################
 =pod
@@ -719,7 +730,7 @@ sub irc_kick
   #:Lasade!bboett@r234m235.cybercable.tm.fr KICK '.$channel.' irc :irc
   #print "restline= ".$line->{"line"}."\n";
   my $botname = $args->{"heap"}->{"botname"};
-  my $msgHand = $this->{"actors"}->{"messageHandler"};
+  my $msgHand = $this->module("messageHandler");
   #my $actLang = $msgHand->fetchChannelLang($channel);
   #$args->{"lang"} = $actLang;
   my $actLang = $args->{"lang"};
@@ -761,7 +772,7 @@ sub irc_part
   my ($this,$subargs) = @_[OBJECT,ARG0];
   $subargs->{"kernel"} = $_[KERNEL];
   #print "irc_part".Dumper(@_);
-  print "irc_part\n";
+  print "zebot::irc_part ".$subargs->{"usernick"}."\n";
   my $success = 0;
   my $theactors = $this->{"actors"};
   foreach my $actorname (keys(%$theactors))
@@ -784,7 +795,7 @@ sub irc_quit
 {
   my ($this,$subargs) = @_[OBJECT,ARG0];
   $subargs->{"kernel"} = $_[KERNEL];
-  print "irc_part\n";
+  print "zebot::irc_quit ".$subargs->{"usernick"}."\n";
   my $success = 0;
   my $theactors = $this->{"actors"};
   foreach my $actorname (keys(%$theactors))
@@ -1040,19 +1051,19 @@ sub connectServer
   { 
     if($this->{"servers"}->{$name})
     {
-    $msg .= $this->getMesg("INSTSERVHAND",$arg)."\n";
-    $this->{"handles"}->{$name} = new zebot::ircServer(); 
-    if($mode)
-    {
-      $this->{"handles"}->{$name}->{"autoconnect"} = 1;
-    }# if($mode)
-    $msg .= $this->getMesg("INITSERVHAND",$arg)."\n";
-    $this->{"handles"}->{$name}->init($name, $this->{"servers"}->{$name},$this);
-    $msg .= $this->getMesg("SERVINSTOK")."\n"; 
+      $msg .= $this->getMesg("INSTSERVHAND",$arg)."\n";
+      $this->{"handles"}->{$name} = new zebot::ircServer(); 
+      if($mode)
+      {
+	$this->{"handles"}->{$name}->{"autoconnect"} = 1;
+      }# if($mode)
+      $msg .= $this->getMesg("INITSERVHAND",$arg)."\n";
+      $this->{"handles"}->{$name}->init($name, $this->{"servers"}->{$name},$this);
+      $msg .= $this->getMesg("SERVINSTOK")."\n"; 
     }# if($this->{"servers"}->{$name})
     else
     {
-    $msg .= $this->getMesg("There is no template for server",$arg)."\n"; 
+      $msg .= $this->getMesg("There is no template for server",$arg)."\n"; 
     }# else
   }# else
   return $msg;
@@ -1170,142 +1181,8 @@ sub loadSettings
     $_[KERNEL]->post( $line->{"context"}, 'privmsg',$line->{"usernick"},"loaded settings ok");
   }# if($_[KERNEL])
 
-
-#
-#   my $basepath = $this->{"settings"}->{"configpath"};
-#   $basepath .= "/" if(!($basepath =~ /\/$/));
-#   #ensure the / at the end the other modules will trust that...
-#   $this->{"settings"}->{"configpath"} = $basepath;
-#   $basepath .= "config/";
-#
-#   my $subsettings = {};
-#
-#   if(opendir(DIRLIST,$basepath))
-#   {
-#     my @dirlisting = grep { /\.conf|\.xml$/ && -f "$basepath/$_" }readdir(DIRLIST);
-#     foreach my $filename (@dirlisting) 
-#     {
-#       if($filename =~ /(.*?)\.conf$/)
-#       {
-#         my $actor = $1;
-#         my $actorSets = {};
-#         $filename = $basepath .$filename;
-#         $this->loadHash($filename,$actorSets);
-#         #print("Loaded for $actor: ".Dumper($actorSets)."\n");
-#         $subsettings->{$actor} = $actorSets;
-#       }# if($filename =~ /\.conf$/)
-#       elsif($filename =~ /(.*?)\.xml$/)
-#       {
-#         print("XML configuration files aren't parsed for the moment\n");
-#       #  my $actor = $1;
-#       #  my $actorSets = {};
-#       #  $actorSets = XMLin($basepath."$filename");
-#       #  #print "dumping config:\n".Dumper($actorSets)."\n";
-#       #  print( "DUMPING config keys:\n".join(' ',keys(%$actorSets))."\n");
-#       #  
-#       }# elsif($filename =~ /(.*?)\.xml$/)
-#     }#foreach $filename (@dirlisting) 
-#
-#     closedir(DIRLIST);
-#   }# if(opendir(DIRLIST,$lpath))
-#   else
-#   {
-#     print("zebot::loadSettings couldn't open the configdir:'$basepath'!!!\n");
-#     die("You didn't read the documentation!!\n");
-#   }# else
-#   # throwing out the hashes of subsettings that aren't needed
-#   my $actorSettings = $subsettings->{"actors"};
-#   my @actornames = keys(%$actorSettings);
-#   $this->{"actornames"} = \@actornames;
-#   #cleanse the actor settings from the records that have no real data
-#   foreach my $name (@actornames)
-#   {
-#     if($actorSettings->{$name} != 1)
-#     {
-#       $this->{"settings"}->{"subsettings"}->{"actors"}->{$name} = $actorSettings->{$name};
-#     }# if($actorSettings->{$name} != 1)
-#   }# foreach my $name (@actornames)
-#   #throw this out of further processing
-#   delete $subsettings->{"actors"};
-#   #merge the zebot stuff with the general settings
-#   foreach my $name (keys(%{$subsettings->{"zebot"}}))
-#   {
-#    $this->{"settings"}->{$name} = $subsettings->{"zebot"}->{$name};
-#   }# foreach my $name (keys(%{$subsettings->{"zebot"}}))
-#   #throw this out of further processing
-#   delete $subsettings->{"zebot"};
-#   #add the remaining stuff to the subsettings
-#   foreach my $subset (keys(%$subsettings))
-#   {
-#     if($subsettings->{$subset} && scalar(keys(%{$subsettings->{$subset}})) >0)
-#     {
-#     $this->{"settings"}->{"subsettings"}->{$subset} = $subsettings->{$subset};
-#     }# if($subsettings->{$subset})
-#   }# foreach my $subset (keys(%$subsettings))
-#   #remove temporary structures
-#   delete $this->{"tmpsettings"};
-#   # Load the irc server settings.... for the moment still a hash...
-#   $this->{"servers"} = do($basepath.'servers.conf');
-#   if($_[KERNEL])
-#   {
-#    my $line = $_[ARG0];
-#     $_[KERNEL]->post( $line->{"context"}, 'privmsg',$line->{"usernick"},"loaded settings ok");
-#   }# if($_[KERNEL])
 }# sub loadSettings 
 #########################################################
-# =pod
-#
-# =item loadHash
-#
-# suppose a file organized line wise with
-# key value
-# open the file, read it in, aprse each line for those key value pairs and add
-# them to the hash of settings
-#
-#
-# if lines are empty or prepended with a # ignore them
-#
-# =cut
-#
-# #########################################################
-# sub loadHash
-# {
-#   my ($this,$filename,$params) = @_;
-#
-#   if(open (SETTINGS, $filename))
-#   {
-#   #print "-- loadSettings over settings ".Dumper($this->{"settings"})."\n";
-#     while(<SETTINGS>)
-#     {
-#       chomp( $_ );
-#       my $tagname;
-#       my $value;
-#       if(/^#/ || /^\s*$/ || /^\s*\{|^\s*\}/)
-#       {
-#         #ignore....
-#       }
-#       elsif(/^(\S+)\s+(.*)$/)
-#       {
-#         $tagname = $1;
-#         $value = $2;
-#         #print "setting $tagname to $value\n";
-#         $value =~ s/\s*#.*$//;
-#         $params->{$tagname} = $value if(!($value =~ /^HASH/));
-#       }#if(/\w+:\w+:\w+.*/)
-#       elsif(/^(\S+)/)
-#       {
-#         $tagname = $1;
-#         $value = 1;
-#         #print "setting $tagname to $value\n";
-#         $params->{$tagname} = $value;
-#       }#if(/\w+:\w+:\w+.*/)
-#     }#while(<SETTINGS>)
-#     close (SETTINGS);
-#   }#if(open (SETTINGS, "zebot.conf"))
-#   else
-#   { print("zebot::loadHash couldn't open $filename\n");}
-# }# sub loadHash
-#########################################################
 =pod
 
 =item dumpSettings 
@@ -1320,6 +1197,7 @@ sub dumpSettings
   my ($this,$hash) = @_;
   my $i;
   my $listing = "";
+  $hash = $this->{"settings"} if(!$hash);
 
   foreach $i (sort(keys(%{$hash})))
   {
@@ -1386,6 +1264,7 @@ sub loadActors
   $this->{"actors"} = {};
 
   #some shortcuts to the array of actornames, settings and actor instances
+  # eval("use zebot::baseactor");
   my $actors = $this->{"actors"};
 
   foreach my $file (keys(%{$this->{"actornames"}}))
@@ -1395,6 +1274,13 @@ sub loadActors
     #TODO don't know how to handle this correctly... i want to forcibly reload the class when loading/reloading the actors, but i don't want to load them twice either....
     #print"using $file.pm\n";
     eval("use zebot::$file");
+    unless( exists $zebot::{$file.'::'})
+    {
+      print "couldn't find lib $file\n";
+      print "trying do....\n";
+      eval("do zebot::$file");
+      print "no way to load lib $file\n" unless exists $zebot::{$file.'::'};
+    }
     #print"instantiating $file\n";
     my $cmd = 'new zebot::'.$file.'()';
     #print "trying to eval $cmd\n";
@@ -1402,6 +1288,7 @@ sub loadActors
     my $subsettings = $this->{"settings"}->{"subsettings"}->{"actors"};
     if($subsettings->{$file})
     {
+      $this->print("Su_BSETTING ARGS FOR $file with :".Dumper($subsettings->{$file}."\n"));
       $newObj->setArgument($subsettings->{$file});
     }# if($subsettings->{"actors"}->{$file})
     #my $newObj = new mailbox() if( $file eq "mailbox" );
@@ -1442,7 +1329,7 @@ sub loadActors
     if($line && $line->{"reloading"})
     {
       #print("RELOADING :args=".Dumper(keys(%$line))."\n");
-	print("RELOAD parsing now handlers \n");
+      print("RELOAD parsing now handlers \n");
       foreach my $net (keys(%{$this->{"handles"}}))
       {
 	my $handler = $this->{"handles"}->{$net};
@@ -1500,15 +1387,15 @@ sub dropActors
     $package .= "::";
 
     delete_package($package);
-    eval("delete $package");
+    #eval("delete $package");
     print "deleted\n" unless exists $zebot::{$file.'::'};
   }#foreach $i (keys(%$this->{"actors"}))
   my $package = "zebot::baseactor";
   eval("no $package");
-    $package .= "::";
-    eval("delete $package");
+  $package .= "::";
+  eval("delete $package");
   delete_package($package);
-    print "deleted\n" unless exists $zebot::{$package};
+  print "deleted\n" unless exists $zebot::{$package};
   #print "deleted baseactor\n" unless exists $zebot::{'baseactor::'};
   ##eval("use $package");
   print("Done unloading \n");
@@ -1584,7 +1471,7 @@ sub getMesg
   my($this,$token,$line) = @_;
   return "toto" if(!$token);
   $line->{"lang"} = $this->setting("language") if(!($line->{"lang"}));
-  my $Msg = $this->{"actors"}->{"messageHandler"};
+  my $Msg = $this->module("messageHandler");
 
   my $msg = $token;
   if($Msg)
@@ -1611,6 +1498,20 @@ settings without disturbing the rest of the program...
 sub setting 
 {
   my ($this,$key,$value) = @_;
+
+  if(!$key)
+  {
+    #make a dump of the settings
+    my $msg = "Settings of Mainframe\n";
+    foreach $key (sort(keys(%{$this->{"settings"}})))
+    {
+      $msg .= "$key: ".$this->{"settings"}->{$key}."\n" 
+	if(ref($this->{"settings"}->{$key}) eq ""); 
+    }# foreach $key (keys(%$this))
+    $msg .= "End of listing\n";
+    return($msg);
+  }# if(!$key)
+
   if(!$value)
   {
     return $this->{"settings"}->{$key};
@@ -1665,6 +1566,42 @@ sub channel_namelist
   $subargs->{"kernel"} = $_[KERNEL];
   print ("irc_channel_namelist\n");
 }
+#########################################################
+=pod
+
+=item language
+
+fetch the language for a given channel, otherwise return the list of installed
+channels and languages
+
+in otherterms setter and getter for the language...
+
+=cut
+
+#########################################################
+sub language
+{
+  my ($this,$channel,$lang) = @_;
+  my $languageRef = $this->{"channelLangs"};
+  if($channel)
+  {
+    $channel = lc($channel);
+    if($lang)
+    {
+      $languageRef->{$channel} = $lang;
+    }# if($lang)
+    return($languageRef->{$channel}); 
+  }# if($arg)
+
+  my $msg;
+  foreach my $i (keys(%$languageRef))
+  {
+    $msg .= "$i ->".$languageRef->{$i}."\n";
+  }#foreach $i (keys(%$this->{"actors"}))
+  $msg .= "...X\n";
+  return $msg;
+}# sub language
+
 1
 __END__
 
